import QtQuick 2.6
import QtQuick.Controls 2.1
import QtQuick.Layouts 1.1

Item {
    id: root
    objectName: "applicationLauncher"

    RowLayout {
        Button {
            text: "Reset"
            onClicked: {
                var idx = _pathViewApplicationLauncher.currentIndex
                _pathViewApplicationLauncher.model = null;
                _pathViewApplicationLauncher.model = _listModel;
                _pathViewApplicationLauncher.currentIndex = idx;
            }
        }
    }

    property int moveAnimationDuration: 500

    ListModel {
        id: _listModel
        ListElement {
            internalName: "Lorem"
            optional: false
        }
        ListElement {
            internalName: "ipsum"
            optional: false
        }
        ListElement {
            internalName: "dolor"
            optional: false
        }
        ListElement {
            internalName: "sit"
            optional: false
        }
        ListElement {
            internalName: "amet"
            optional: false
        }
        ListElement {
            internalName: "consectetur"
            optional: false
        }

        onRowsAboutToBeMoved: {
            console.info("Rows about to be moved", sourceStart, sourceEnd, destinationRow)
        }

        onRowsMoved: {
            console.info("Rows moved", start, end, row)
        }

        onRowsAboutToBeInserted: {
            console.info("Rows about to be inserted", first)
        }

        onRowsInserted: {
            console.info("Rows inserted", first)
        }

        onRowsAboutToBeRemoved: {
            console.info("Rows about to be removed", first, last)
        }

        onRowsRemoved: {
            console.info("Rows removed", first, last)
        }
    }

    ListView {
        id: _pathViewApplicationLauncher

        readonly property int delegateWidth: parent.width / 3
        property int directionOfLastKeyInput: -1
        property bool isMovementActive: false

        model: _listModel

        focus: true



        anchors.fill: parent
        anchors.topMargin: 60

        orientation: ListView.Horizontal

        highlightRangeMode: ListView.StrictlyEnforceRange
        preferredHighlightBegin: (0.5 * width) - 0.5*delegateWidth
        preferredHighlightEnd: (0.5 * width) + 0.5*delegateWidth

        highlightMoveDuration: 350

        snapMode: ListView.SnapToItem
        flickDeceleration: 5500

        cacheBuffer: width

        Keys.onLeftPressed: {
            decrementCurrentIndex()
        }

        Keys.onRightPressed: {
            incrementCurrentIndex()
        }

        removeDisplaced: Transition {
            NumberAnimation { properties: "x"; duration: root.moveAnimationDuration; easing.type: Easing.OutQuad }
        }

        addDisplaced: Transition {
            NumberAnimation { properties: "x"; duration: root.moveAnimationDuration; easing.type: Easing.OutQuad }
        }

        displaced: Transition {
            NumberAnimation { properties: "x"; duration: root.moveAnimationDuration; easing.type: Easing.OutQuad }
        }

        move: Transition {
            NumberAnimation { properties: "x"; duration: root.moveAnimationDuration; easing.type: Easing.OutQuad }
        }

        currentIndex: 1


        delegate: Item {
            id: delegateRoot

            readonly property bool isCurrentItem: ListView.isCurrentItem
            readonly property ListView view: ListView.view

            width: _pathViewApplicationLauncher.delegateWidth
            height: _pathViewApplicationLauncher.delegateWidth

            Rectangle {
                anchors.fill: parent
                anchors.margins: 8
                border.color: isCurrentItem ? "#00ffff" : "#ffcc00"
                border.width: isCurrentItem ? 8 : 2
                color: "#200000ff"
                Text {
                    anchors.centerIn: parent
                    color: "#fff"
                    text: internalName
                    font.pixelSize: 40
                }
                Rectangle {
                    width: 80
                    height: 40
                    y: index * 40
                    color: "#ffd000"
                    Text {
                        anchors.fill: parent
                        font.pixelSize: 12
                        verticalAlignment: Text.AlignVCenter
                        horizontalAlignment: Text.AlignHCenter
                        text: "Index: " + index + "<br/>x: " + delegateRoot.x
                    }
                }
            }
        } //delegateRoot
    }

    Item {
        id: _rowSlabs
        property int delegateWidth: 80
        width: childrenRect.width
        height: 50

        anchors.horizontalCenter: parent.horizontalCenter

        Repeater {
            width: _rowSlabs.delegateWidth * count
            model: _listModel
            delegate: Rectangle {
                x: width * index
                width: _rowSlabs.delegateWidth
                height: 50
                border.width: 4
                border.color: _pathViewApplicationLauncher.currentIndex === index ? "#a0f" : "#000"

                Text {
                    anchors.fill: parent
                    anchors.margins: 4
                    text: "<b>I:%1</b><br/>%2".arg(index).arg(internalName)
                }
            }
        }
    }

    Timer {
        id: _hideTimer
        interval: 1000
        running: true
        onTriggered: {
            _listModel.append({internalName: "adipiscing", optional: true});
            _listModel.append({internalName: "elit", optional: true});
            _listModel.append({internalName: "sed", optional: true});
            _listModel.append({internalName: "do", optional: true});
            _listModel.append({internalName: "eiusmod", optional: true});
        }
    }
}
